#!/usr/bin/env python3

from serial import Serial
from serial.tools.list_ports import comports as port_list

from remotes import tv

from pprint import pprint
from time import sleep

ser = Serial(timeout=1)
ser.baudrate = 9600
#ser.port = '/dev/ttyACM0'

# Find the first Arduino we can find (Or verify the port we got is valid)
ports_open = port_list(False)
found = False
for p in ports_open:
    print("Device: /dev/{0:8}  Manufacturer: {1}".format(p.name, p.manufacturer))
    if ser.port == None:
        if p.manufacturer == "Arduino (www.arduino.cc)":
            found = True
            ser.port = "/dev/{0}".format(p.name)
            print("Automagically found an Arduino on port '/dev/{0}'!".format(p.name))
            break # Stop needlessly looping over devices
    else:
        if '/dev/{0}'.format(p.name) == ser.port:
            found = True
            print("Found your Arduino!")
            break # Stop needlessly looping over devices

# Verify I have found a Arduino
if not found:
    raise TypeError("Device '{0}' not found!".format(ser.port))

# Attempts to send the requested code
def send_code(ky):
    if ky in tv:
        msg = "0x{0},{1},1\n".format(tv[ky], tv["_config"]["size"])
        ser.write(msg.encode())
        #print("Sent {0} {1}".format(ky, msg.encode()))
    else:
        print("Invalid key")

class Navigate():
    """ A collection of orders/commands to be sent """
    def __init__(self):
        """ Initialize with no orders """
        self.orders = []
    
    def add_order(self, order, delay=2, repeat=1):
        """ Adds a new order to the end of the list of orders
            Given:
                Order
                Delay in seconds
                If wanted you can issue the command multiple times (Que it up in a single add_order)
        """
        for x in range(0, repeat):
            self.orders.append({"order": order, "delay": delay})
    
    def perform(self, response):
        """ Sends the code then waits, executes the first then removes it """
        if len(self.orders) > 0:
            if response in ("Ready!", "Ok"):
                send_code(self.orders[0]["order"])
                sleep(self.orders[0]["delay"])
                del self.orders[0]
                #print("There are {0} remaining orders".format(len(self.orders)))

pluto = Navigate()
pluto.add_order("power", delay=7)
pluto.add_order("home", repeat=2)
pluto.add_order("right", repeat=3)
pluto.add_order("down", repeat=8)
pluto.add_order("mute")
pluto.add_order("ok")

# Ok we are ready to actually open the Serial port
ser.open()
print("Opened!")
while ser.is_open: # While the connection is open
    line = ser.readline().decode().strip("\n").strip("\r") # Attempt to read a line
    if line != "": # If there is a line Pretty Print it to the screen
        #pprint(line)
        pluto.perform(line)
    if len(pluto.orders) <= 0:
        ser.close()
        print("Closed!")
